@@ -38,6 +38,7 @@
 #' of asymmetry
 #' @param mesh A mesh3d object to be warped to represent shape deformation of the directional and fluctuating components
 #' of asymmetry if {warpgrids= TRUE} (see \code{\link{warpRefMesh}}).
+#' @param ShowPlot A logical value indicating whether or not the plot should be returned
 #' @param verbose A logical value indicating whether the output is basic or verbose (see Value below)
 #' @keywords analysis
 #' @export
@@ -64,7 +65,7 @@
 #' data(scallops)
 #' bilat.symmetry(scallops$coorddata,ind=scallops$ind,object.sym=TRUE,land.pairs=scallops$land.pairs)
 bilat.symmetry<-function(A,ind=NULL,side=NULL,replicate=NULL,object.sym=FALSE,land.pairs=NULL,
-      warpgrids = TRUE, mesh=NULL, verbose =FALSE){
+      warpgrids = TRUE, mesh=NULL, verbose =FALSE,ShowPlot=TRUE){
   if (length(dim(A))!=3){
     stop("Data matrix not a 3D array (see 'arrayspecs').")  }
   if(any(is.na(A))==T){
@@ -79,11 +80,22 @@ bilat.symmetry<-function(A,ind=NULL,side=NULL,replicate=NULL,object.sym=FALSE,la
     side<-as.factor(side)
     gpa.res<-gpagen(A,ShowPlot=FALSE)
     shape<-two.d.array(gpa.res$coords)
-    f1<-"shape~ind*side"; if(!is.null(replicate)){f1<-paste(f1,"ind:side:replicate",sep="+")}
+    f1<-"shape~ind*side"
+    f1.df<-data.frame(shape=shape,ind=ind,side=side)
+    if(!is.null(replicate)){
+      f1<-paste(f1,"ind:side:replicate",sep="+")
+      f1.df<-data.frame(shape=shape,ind=ind,side=side, replicate=replicate)}
     f1<-as.formula(f1)
-    f2<-"gpa.res$Csize~ind*side"; if(!is.null(replicate)){f2<-paste(f2,"ind:side:replicate",sep="+")}
+    pf1<-procD.fit(f1, data=f1.df)
+    f2<-"gpa.res$Csize~ind*side"
+    f2.df<-data.frame(Csize=gpa.res$Csize, side=side, ind=ind)
+    if(!is.null(replicate)){
+      f2<-paste(f2,"ind:side:replicate",sep="+")
+      f2.df<-data.frame(Csize=gpa.res$Csize, side=side, ind=ind, replicate=replicate)
+    } 
     f2<-as.formula(f2)
-    res.shape<-anova.parts(f1,Yalt="observed")$table    
+    pf2<-procD.fit(f2, data=f2.df)
+    res.shape<-anova.parts(pf1, keep.order=FALSE,Yalt="observed")$table    
     res.shape<-res.shape[1:(dim(res.shape)[1]-2),1:(dim(res.shape)[2]-2)]
     res.shape[1,1]<-(nind-1)*shpsp;res.shape[2,1]<-shpsp; res.shape[3,1]<-(nind-1)*shpsp
     if(!is.null(replicate)){res.shape[4,1]<-(nrep-1)*nind*2*shpsp}
@@ -118,33 +130,35 @@ bilat.symmetry<-function(A,ind=NULL,side=NULL,replicate=NULL,object.sym=FALSE,la
     asymm.component<-array(data=NA,dim=c(p,k,n))     
       dimnames(symm.component)[[3]] <- dimnames(asymm.component)[[3]]<-spec.names
     for (i in 1:n){ asymm.component[,,i]<-(gpa.res$coords[,,i]-symm.component[,,i]) + mn.shape}
-      if(warpgrids == TRUE){
-        if(k==2){  
-          par(mfrow=c(2,2),oma=c(1.5,0,1.5,0))
-          plotAllSpecimens(symm.component)
-          plotAllSpecimens(asymm.component)
-          plotRefToTarget(DA.mns[,,1],DA.mns[,,2],method="TPS",main="Directional Asymmetry")
-          plotRefToTarget(FA.mns[,,1],FA.mns[,,2],method="TPS",main="Fluctuating Asymmetry")
-          mtext("Symmetric Shape Component (left) and Asymmetric Shape Component (right)",outer = TRUE,side=3)
-          mtext("Mean directional (left) and fluctuating (right) asymmetry",side = 1, outer = TRUE)
-          par(mfrow=c(1,1))
+      if(ShowPlot==TRUE){ 
+        if(warpgrids == TRUE){
+          if(k==2){  
+            par(mfrow=c(2,2),oma=c(1.5,0,1.5,0))
+            plotAllSpecimens(symm.component)
+            plotAllSpecimens(asymm.component)
+            plotRefToTarget(DA.mns[,,1],DA.mns[,,2],method="TPS",main="Directional Asymmetry")
+            plotRefToTarget(FA.mns[,,1],FA.mns[,,2],method="TPS",main="Fluctuating Asymmetry")
+            mtext("Symmetric Shape Component (left) and Asymmetric Shape Component (right)",outer = TRUE,side=3)
+            mtext("Mean directional (left) and fluctuating (right) asymmetry",side = 1, outer = TRUE)
+            par(mfrow=c(1,1))
+          }
+          if (k==3){
+            if (is.null(mesh)==TRUE){
+              open3d()
+              plotRefToTarget(DA.mns[,,1],DA.mns[,,2],method="points",main="Directional Asymmetry")
+              open3d()
+              plotRefToTarget(FA.mns[,,1],FA.mns[,,2],method="points",main="Fluctuating Asymmetry")
+              } 
+            if(is.null(mesh)==FALSE){
+              plotRefToTarget(DA.mns[,,1],DA.mns[,,2],mesh,method="surface")
+              title3d(main="Directional Asymmetry")
+              plotRefToTarget(FA.mns[,,1],FA.mns[,,2],mesh,method="surface")
+              title3d(main="Fluctuating Asymmetry")
+              }
+          }
+        layout(1) 
         }
-        if (k==3){
-          if (is.null(mesh)==TRUE){
-            open3d()
-            plotRefToTarget(DA.mns[,,1],DA.mns[,,2],method="points",main="Directional Asymmetry")
-            open3d()
-            plotRefToTarget(FA.mns[,,1],FA.mns[,,2],method="points",main="Fluctuating Asymmetry")
-            } 
-          if(is.null(mesh)==FALSE){
-            plotRefToTarget(DA.mns[,,1],DA.mns[,,2],mesh,method="surface")
-            title3d(main="Directional Asymmetry")
-            plotRefToTarget(FA.mns[,,1],FA.mns[,,2],mesh,method="surface")
-            title3d(main="Fluctuating Asymmetry")
-            }
-        }
-      layout(1) 
-      }
+      }  
     if(verbose==TRUE){
     	class(res.size) <- c("anova", class(res.size))
     	class(res.shape) <- c("anova", class(res.shape))
@@ -171,9 +185,14 @@ bilat.symmetry<-function(A,ind=NULL,side=NULL,replicate=NULL,object.sym=FALSE,la
     ind<-rep(ind,2);side<-gl(2,n); if(!is.null(replicate)){replicate<-rep(replicate,2)}
     gpa.res<-gpagen(A,ShowPlot = FALSE)
     shape<-two.d.array(gpa.res$coords)    
-    f1<-"shape~ind*side"; if(!is.null(replicate)){f1<-paste(f1,"ind:side:replicate",sep="+")}
+    f1<-"shape~ind*side"; f1.df<-data.frame(shape=shape, ind=ind, side=side)
+    if(!is.null(replicate)){
+      f1<-paste(f1,"ind:side:replicate",sep="+")
+      f1.df<-data.frame(shape=shape, ind=ind, side=side, replicate=replicate)
+      }
     f1<-as.formula(f1)
-    res.shape<-anova.parts(f1,Yalt="observed")$table    
+    pf1<-procD.fit(f1, data=f1.df)
+    res.shape<-anova.parts(pf1, keep.order=FALSE, data=f1.df,Yalt="observed")$table    
     res.shape<-res.shape[1:(dim(res.shape)[1]-2),1:(dim(res.shape)[2]-2)]
     res.shape[,2]<-res.shape[,2]/2 
     res.shape[,2]<-res.shape[,2]/2 
@@ -203,32 +222,35 @@ bilat.symmetry<-function(A,ind=NULL,side=NULL,replicate=NULL,object.sym=FALSE,la
       dimnames(symm.component)[[3]] <- dimnames(asymm.component)[[3]]<-spec.names
     mn.shape<-mshape(gpa.res$coords)
     for (i in 1:n){ asymm.component[,,i]<-(gpa.res$coords[,,i]-symm.component[,,i]) + mn.shape}
-    if(warpgrids==TRUE){
-        if(k==2){  
-          par(mfrow=c(2,2),oma=c(1.5,0,1.5,0))
-          plotAllSpecimens(symm.component)
-          plotAllSpecimens(asymm.component)
-          plotRefToTarget(DA.mns[,,1],DA.mns[,,2],method="TPS",main="Directional Asymmetry")
-          plotRefToTarget(FA.mns[,,1],FA.mns[,,2],method="TPS",main="Fluctuating Asymmetry")
-          mtext("Symmetric Shape Component (left) and Asymmetric Shape Component (right)",outer = TRUE,side=3)
-          mtext("Mean directional (left) and fluctuating (right) asymmetry",side = 1, outer = TRUE)
-        }
-        if (k==3){
-          if(is.null(mesh)==TRUE){
-            open3d()
-            plotRefToTarget(DA.mns[,,1],DA.mns[,,2],method="points",main="Directional Asymmetry")
-            open3d()
-            plotRefToTarget(FA.mns[,,1],FA.mns[,,2],method="points",main="Fluctuating Asymmetry")
-          } 
-          if(is.null(mesh)==FALSE){
-            plotRefToTarget(DA.mns[,,1],DA.mns[,,2],mesh,method="surface")
-            title3d(main="Directional Asymmetry")
-            plotRefToTarget(FA.mns[,,1],FA.mns[,,2],mesh,method="surface")
-            title3d(main="Fluctuating Asymmetry")
-          }  
-        }
-      layout(1) 
+    if(ShowPlot==TRUE){ 
+        if(warpgrids==TRUE){
+          if(k==2){  
+            par(mfrow=c(2,2),oma=c(1.5,0,1.5,0))
+            plotAllSpecimens(symm.component)
+            plotAllSpecimens(asymm.component)
+            plotRefToTarget(DA.mns[,,1],DA.mns[,,2],method="TPS",main="Directional Asymmetry")
+            plotRefToTarget(FA.mns[,,1],FA.mns[,,2],method="TPS",main="Fluctuating Asymmetry")
+            mtext("Symmetric Shape Component (left) and Asymmetric Shape Component (right)",outer = TRUE,side=3)
+            mtext("Mean directional (left) and fluctuating (right) asymmetry",side = 1, outer = TRUE)
+          }
+          if (k==3){
+            if(is.null(mesh)==TRUE){
+              open3d()
+              plotRefToTarget(DA.mns[,,1],DA.mns[,,2],method="points",main="Directional Asymmetry")
+              open3d()
+              plotRefToTarget(FA.mns[,,1],FA.mns[,,2],method="points",main="Fluctuating Asymmetry")
+            } 
+            if(is.null(mesh)==FALSE){
+              plotRefToTarget(DA.mns[,,1],DA.mns[,,2],mesh,method="surface")
+              title3d(main="Directional Asymmetry")
+              plotRefToTarget(FA.mns[,,1],FA.mns[,,2],mesh,method="surface")
+              title3d(main="Fluctuating Asymmetry")
+            }  
+          }
+        layout(1) 
       } 
+    
+    }
     if(verbose==TRUE){
     	class(res.shape) <- c("anova", class(res.shape))
     	return(list(symm.shape=symm.component,asymm.shape=asymm.component, 
