@@ -19,7 +19,7 @@
 #' 
 #' If there are curves defined in the file (i.e., CURVES= fields), the option readcurves should be used.
 #' When readcurves = TRUE, the coordinate data for the curves will be returned as semilandmarks and will be appended to
-#' the fixed landmark data. Then the user needs to use \code{\link{define.sliders.2d}} or \code{\link{define.sliders.3d}}
+#' the fixed landmark data. Then the user needs to use \code{\link{define.sliders}} or \code{\link{define.sliders}}
 #' to create a matrix designating how the curve points will slide (used by 'curves=' in \code{\link{gpagen}}).
 #' When readcurves = FALSE, only the landmark data are returned.
 #' 
@@ -99,7 +99,8 @@ readland.tps <- function (file, specID = c("None", "ID", "imageID"),
   imscale <- aperm(array(rep(imscale, p * k), c(n, k, p)), 
                    c(3, 2, 1))
   coords <- coords * imscale
-  
+  if (readcurves==F){coords<-coords[1:nland,,] 
+      if(n==1) coords <- array(coords, c(nland,k,n))}
   if (specID == "None") {
       if (warnmsg == T) {print("No Specimen names extracted")
     }
@@ -125,7 +126,6 @@ readland.tps <- function (file, specID = c("None", "ID", "imageID"),
       }
     } 
   }
-  
   if (specID == "ID") {
     ID <- sub("ID=", "", tpsfile[grep("ID", tpsfile, ignore.case)], ignore.case)
     if (length(ID) == 0) {
@@ -140,6 +140,5 @@ readland.tps <- function (file, specID = c("None", "ID", "imageID"),
       }
     }
   }
-if (readcurves==F){coords<-coords[1:nland,,]}    
 return(coords = coords)                    
 }
